#define LOGGER

#include <logger.h>
#include <syslog.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>

void init_logger(void) {
   if (loggingmethod & LOGGER_SYSLOG) {
      openlog("gossip", 0, LOG_DAEMON);
   }

   if (loggingmethod & LOGGER_FILE) {
      if ((logfd=fopen(logfile, "a+"))==NULL) {
         perror("Could not open logfile for writing, exiting");
	 exit(1);
      }
   }
}


void log(int severity, const char *fmt, ...) {
   char *timestring;
   char buf[1024];
   time_t now;
   char severitystring[12];
   char *s, *msg;
   char logstring[512];
   va_list ap;
	
   /* If message is debug-message and debugging is disabled, don't log; */
   /* If loggingmethod is 'none', don't log either			*/
   if ((severity==LOGGER_DEBUG && !do_debug) || loggingmethod==LOGGER_NONE) return; 

   /* Build a logstring from the formatstring and other args */
   va_start(ap,fmt);
   vsnprintf(logstring,sizeof(logstring),fmt,ap);
   va_end(ap);
	
   /* Filter some crap */
   msg=strdup(logstring);
   for(s=msg;*s;s++) {
      if (*s == '\n' || *s == '\r' || *s == '\f') *s = 0;
      if (*s == '\b') *s = ' ';
   }

   /* Write logline to stdout */
   printf("%s\n",logstring);
	
   if (loggingmethod & LOGGER_SYSLOG) {
      switch (severity) {
         case LOGGER_DEBUG :
	    syslog(LOG_DEBUG,"%s", msg);
	    break;
	 case LOGGER_INFO :
	    syslog(LOG_INFO,"%s", msg);
	    break;
	 case LOGGER_WARNING : 
	    syslog(LOG_WARNING, "%s", msg); 
	    break;
	 case LOGGER_ERROR :
	    syslog(LOG_ERR, "%s", msg);
	    break;
	 default : 
	    log(LOGGER_ERROR, "The logger was given a wrong severity-level, this should not happen");
	 }
   } 
	
   if (loggingmethod & LOGGER_FILE) {
      time(&now);
      timestring = ctime(&now);
		
      /* Filter out the \n ctime leaves behind */
      timestring[strlen(timestring)-1]='\0';
		
      switch (severity) {
         case LOGGER_DEBUG :
	    strcpy(severitystring,"DEBUG: ");
            break;
	 case LOGGER_INFO :
	    strcpy(severitystring,"INFO: ");
	    break;
	 case LOGGER_WARNING :
	    strcpy(severitystring,"WARNING: ");
	    break;
	 case LOGGER_ERROR :
	    strcpy(severitystring,"ERROR: ");
	    break;
	 default :
	    log(LOGGER_ERROR,"The logger was given a wrong severity-level, this should not happen");
      }

      /* Construct string to be written to file */
      snprintf(buf,sizeof(buf),"%s %s%s\n",timestring, severitystring, msg);

      if ((fwrite(buf, strlen(buf),1 ,logfd))<strlen(buf)) {
         if (loggingmethod & LOG_SYSLOG)
            syslog(LOG_ERR, "There was an error writing to the logfile %s", logfile);
	 }
	 fflush(logfd);
      }

   /* Free malloc()ed mem */
   free(msg);
}

void stop_logger(void) {
   int temp = loggingmethod;

   loggingmethod=LOGGER_NONE;
   if (temp & LOGGER_SYSLOG)
      closelog();
   if (temp & LOGGER_FILE)
      fclose(logfd);
}

